// ------------------------------------------------------------------------------
using System;
using System.Runtime.CompilerServices;
using System.Linq;
using System.Collections.Generic;
using DataContracts;
using System.Text.RegularExpressions;

namespace Ato.EN.IntegrationServices.CodeGenerationIITRPRFL
{

    public partial class IITRPRFL2022Validator
    {
        /// <summary>
        /// The max parameter name length is restricted by the SBR1 and SBR2 schemas - the lowest common denominator of SBR 1 is used as the default
        /// </summary>
        private int _maxParameterNameLength;

        /// <summary>
        /// The max parameter value length is restricted by the SBR1 and SBR2 schemas
        /// </summary>
        private int _maxParameterValueLength;

        /// <summary>
        /// The SBR1 and ebms3 schemas do not allow parameter names or values to be the empty string
        /// </summary>
        private string _emptyParameterValue;

        /// <summary>
        /// The report parameter
        /// </summary>
        private IITRPRFL2022 report;

        /// <summary>
        /// The response parameter
        /// </summary>
        public List<ProcessMessageDocument> response {get; private set;}

        /// <summary>
        /// Initializes a new instance of the <see cref="IITRPRFL2022Validator" /> class.
        /// </summary>
        /// <param name="maxParameterNameLength">Maximum length of the parameter name.</param>
        /// <param name="maxParameterValueLength">Maximum length of the parameter value.</param>
        /// <param name="emptyParameterValue">This value will be used in place of any parameter values that result in a null or empty value.</param>
        public IITRPRFL2022Validator(int maxParameterNameLength = 20, int maxParameterValueLength = 4096, string emptyParameterValue = "EMPTY")
        {
            _maxParameterNameLength = maxParameterNameLength;
            _maxParameterValueLength = maxParameterValueLength;
            _emptyParameterValue = emptyParameterValue;
            this.response = new List<ProcessMessageDocument>();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="IITRPRFL2022Validator" /> class.
        /// </summary>
        /// <param name="report">Report variable </param>                
        /// <param name="maxParameterNameLength">Maximum length of the parameter name.</param>
        /// <param name="maxParameterValueLength">Maximum length of the parameter value.</param>
        /// <param name="emptyParameterValue">This value will be used in place of any parameter values that result in a null or empty value.</param>
        public IITRPRFL2022Validator(IITRPRFL2022 reportIn, int maxParameterNameLength = 20, int maxParameterValueLength = 4096, string emptyParameterValue = "EMPTY"):this(maxParameterNameLength,maxParameterValueLength,emptyParameterValue)
        {                   
            this.ConsumedReport = reportIn;                    
        }

        #region Functions
        private static IEnumerable<string> Union(IEnumerable<string> list1, IEnumerable<string> list2)
        {
            IEnumerable<string> response;

            if (list1 == null && list2 == null)
            {
                response = null;
            }
            else if (list1 == null)
            {
                response = list2.Distinct();
            }
            else if (list2 == null)
            {
                response = list1.Distinct();
            }
            else
            {
                response = list1.Union(list2);
            }

            return response;
        }

        private static bool IsMatch(int? field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(Convert.ToString(field.Value), expression, options);
        }

        private static bool IsMatch(string field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(field, expression, options);
        }

        // This is just context and tuple counts where they are integer values - easier that changing the parsing logic to just return the value
        private static int Count(int count)
        {
            return count;
        }


        private static int Count<T>(IEnumerable<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }


        private static int Count<T>(ICollection<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }

        private static bool exists(bool value)
        {
            return value;
        }

        private static string GetValueOrEmpty(bool? val)
        {
            return (val.HasValue) ? val.ToString().ToLower() : string.Empty;
        }

        private static string GetValueOrEmptyDate(DateTime? val)
        {
            return (val.HasValue) ? val.Value.ToString("yyyy-MM-dd") : string.Empty;
        }

        private static string GetValueOrEmptyDate(string val)
        {
            DateTime dateTimeOut;
            if (DateTime.TryParse(val, out dateTimeOut))
            {
                return dateTimeOut.ToString("yyyy-MM-dd");
            }
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmptyDateTime(string val)
        {
            DateTime dateTimeOut;
            if (DateTime.TryParse(val, out dateTimeOut))
            {
                return dateTimeOut.ToString("yyyy-MM-ddTHH:mm:ssZ");
            }
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmptyDateTime(DateTime? val)
        {
            return (val.HasValue) ? val.Value.ToString("yyyy-MM-ddTHH:mm:ssZ") : string.Empty;
        }

        private static string GetValueOrEmpty(string val)
        {
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmpty(decimal? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static string GetValueOrEmpty(int? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

         /// <summary>
         /// Get string value between [first] a and [last] b.
         /// </summary>
        public static string Between(string value, string a, string b)
        {
            int posA = value.IndexOf(a);
            int posB = value.LastIndexOf(b);
            if (posA == -1)
            {
                return "";
            }
            if (posB == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= posB)
            {
                return "";
            }
            return value.Substring(adjustedPosA, posB - adjustedPosA);
        }

         /// <summary>
         /// Get string value after [first] a.
         /// </summary>
        public static string Before(string value, string a)
        {
            int posA = value.IndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            return value.Substring(0, posA);
        }

         /// <summary>
         /// Get string value after [last] a.
         /// </summary>
        public static string After(string value, string a)
        {
            int posA = value.LastIndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= value.Length)
            {
                return "";
            }
            return value.Substring(adjustedPosA);
        }

        private static int Length(object field)
        {
            if (field == null)
            return 0;
            else
            return field.ToString().Trim().Length;
        }

        private static bool NotSameValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool NotSameValues(params object[] nodes)
        {
            if (nodes == null)
                return false;

            return (from x in nodes select x).Distinct().Count() != 1;
        }

        private static bool HasDuplicateValues(IEnumerable<DateTime> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool HasDuplicateValues(IEnumerable<DateTime?> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool NotSameValues(IEnumerable<DateTime> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool NotSameValues(IEnumerable<DateTime?> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool HasDuplicateValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool HasDuplicateValues(params object[] nodes)
        {
           if (nodes == null)
                return false;

            nodes = nodes.Where(x => x != null).ToArray();
            return !((from x in nodes select x).Distinct().Count() == nodes.Count());
        
        }

        private int DuplicateValueIndex(IEnumerable<object> values)
        {
            int response = 0;
            var hashset = new HashSet<object>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private int DuplicateValueIndex<T>(IEnumerable<T?> values) where T : struct
        {
            int response = 0;
            var hashset = new HashSet<T?>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private static bool IsDate(object value)
        {
            DateTime dateValue;
            return (value != null && DateTime.TryParse(value.ToString(), out dateValue));
        }

        public DateTime AsDate(string dateAsString)
        {
            DateTime response = DateTime.MinValue;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date;
            }

            return response;
        }

        public DateTime Date(DateTime? datetime)
        {                   
            return datetime.GetValueOrDefault().Date;
        }

				//The logic in After function expects "---" for day and "--" for month. 
				//Since hyphen was missing the date always returned null
        public DateTime? ConvertToDate(int day, int month, int year)
        {
					return ConvertToDate(day == 0 ? null : "---" + day.ToString(), month == 0 ? null : "--" + month.ToString(), year == 0 ? null : year.ToString());
        }

        public DateTime? ConvertToDate(string day, string month, string year)
        {
            DateTime? response;
            DateTime result;

            if (year == null || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        public DateTime? ConvertToDate(string day, string month, int year)
        {
            DateTime? response;
            DateTime result;

            if (year == 0 || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year.ToString() + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        private static int Day(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Day;
            }

            return response;
        }


        private static int Day(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Day;

        }


        private static string Month(string dateAsString)
        {
            string response = null;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.ToString("MMMM");
            }
            else
            {
                return "NotAMonth";
            }

            return response;
        }

        private static string Month(DateTime? date)
        {
            if (date == null)
                return "NotAMonth";
            else
                return date.Value.ToString("MMMM");
        }


        private static int MonthAsInt(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Month;
            }

            return response;
        }

        private static int MonthAsInt(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            return date.Value.Month;
        }


        private static int Year(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Year;
            }

            return response;
        }

        private static int Year(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Year;
        }


        private static int CurrentFinancialYear()
        {
            return DateToFinancialYear(DateTime.Now, 7);
        }

        private static int FinancialYear(string dateAsString)
        {
            return DateToFinancialYear(dateAsString, 7);
        }

        private static int FinancialYear(DateTime? date)
        {
            return DateToFinancialYear(date, 7);
        }

        private static int DateToFinancialYear(string dateAsString, int startingMonth)
        {
            int response = 0;
            DateTime date;
            if (DateTime.TryParse(dateAsString, out date))
            {
                response = DateToFinancialYear(date, startingMonth);
            }

            return response;
        }
        private static int DateToFinancialYear(DateTime? date, int startingMonth)
        {
            int response;
            if (date == null)
            {
                response = 0;
            }
            else
            {
                int year = date.Value.Year;
                int month = date.Value.Month;

                if (startingMonth > month)
                    response = year;
                else
                    response = year + 1;
            }
            return response;
        }


        private static int FBTYear(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
                response = FBTYear(date);
            return response;
        }

        private static int FBTYear(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            else
            {
                if (date.Value.Month > 3)
                    return date.Value.Year + 1;
                else
                    return date.Value.Year;
            }
        }

        private static DateTime? AddMonthsToDate(DateTime? dateTime, int months)
        {
            return dateTime == null ? null : (DateTime?)dateTime.Value.AddMonths(months);
        }

        private static bool IsNumeric(object value)
        {
            decimal numbervalue;
            return (value != null && decimal.TryParse(value.ToString(), out numbervalue));
        }

        private static bool NotMonetary(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(int? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }

        private static bool NotNumeric(long? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }


        private static bool OutsideRange(decimal field, decimal expression, int range)
        {
            bool response;

            response = (field < (expression - range)) || (field > (expression + range));

            return response;
        }


        private static bool FailsUSIAlgorithm(string usi, string abn)
        {
            bool response = true;
            if (usi == null || abn == null)
            {
                response = false;
            }
            else
            {
                usi = usi.Trim();
                abn = abn.Trim();
                if (usi.Length == 14)
                {
                    int numeric;
                    if (usi.Substring(0, 11) == abn && int.TryParse(usi.Substring(11, 3), out numeric))
                        response = false;

                }
                else if (usi.Length == 9)
                {

                    if (Regex.IsMatch(usi, @"^([a-zA-Z]{3}\d{4}[a-zA-Z]{2})$"))
                        response = false;
                }
            }
            return response;
        }

        private static bool FailsTANAlgorithm(string tan)
        {
            bool response;
            decimal decimalTan;

            if (tan == null)
                return false;

            tan = tan.Trim();

            if (!decimal.TryParse(tan, out decimalTan))
                return true;

            if (tan.Length != 8)
                return true;

            decimal tanSum =
                7 * int.Parse(tan.Substring(0, 1)) +
                9 * int.Parse(tan.Substring(1, 1)) +
                8 * int.Parse(tan.Substring(2, 1)) +
                4 * int.Parse(tan.Substring(3, 1)) +
                6 * int.Parse(tan.Substring(4, 1)) +
                3 * int.Parse(tan.Substring(5, 1)) +
                5 * int.Parse(tan.Substring(6, 1)) +
                1 * int.Parse(tan.Substring(7, 1));

            if ((tanSum % 11) == 0)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsABNAlgorithm(string abn)
        {
            bool response;
            decimal decimalAbn;

            if (abn == null)
                return false;

            abn = abn.Trim();

            if (!decimal.TryParse(abn, out decimalAbn))
                return true;

            if (abn.Length != 11)
                return true;

            decimal abnSum =
                10 * (int.Parse(abn.Substring(0, 1)) - 1) +
                1 * int.Parse(abn.Substring(1, 1)) +
                3 * int.Parse(abn.Substring(2, 1)) +
                5 * int.Parse(abn.Substring(3, 1)) +
                7 * int.Parse(abn.Substring(4, 1)) +
                9 * int.Parse(abn.Substring(5, 1)) +
                11 * int.Parse(abn.Substring(6, 1)) +
                13 * int.Parse(abn.Substring(7, 1)) +
                15 * int.Parse(abn.Substring(8, 1)) +
                17 * int.Parse(abn.Substring(9, 1)) +
                19 * int.Parse(abn.Substring(10, 1));

            if ((abnSum % 89) == 0)
                response = false;
            else
                response = true;

            return response;
        }
        private static bool FailsACNAlgorithm(string acn)
        {
            bool response;
            decimal decimalAbn;
            if (acn == null)
                return false;

            acn = acn.Trim();

            if (!decimal.TryParse(acn, out decimalAbn))
                return true;

            if (acn.Length != 9)
                return true;

            decimal abnSum =
                8 * int.Parse(acn.Substring(0, 1)) +
                7 * int.Parse(acn.Substring(1, 1)) +
                6 * int.Parse(acn.Substring(2, 1)) +
                5 * int.Parse(acn.Substring(3, 1)) +
                4 * int.Parse(acn.Substring(4, 1)) +
                3 * int.Parse(acn.Substring(5, 1)) +
                2 * int.Parse(acn.Substring(6, 1)) +
                1 * int.Parse(acn.Substring(7, 1));

            decimal checkDigit = int.Parse(acn.Substring(8, 1));
            decimal acnRemainder = abnSum % 10;

            if (((10 - acnRemainder) % 10) == checkDigit)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsTFNAlgorithm(string tfn)
        {
            bool response;
            decimal decimalTfn;

            if (tfn == null)
                return false;

            tfn = tfn.Trim();
            tfn = Regex.Replace(tfn, "^0+", "");

            if (!decimal.TryParse(tfn, out decimalTfn))
                return true;

            if (tfn.Length < 8)
                return true;


            decimal tfn1To7Sum =
                1 * int.Parse(tfn.Substring(0, 1)) +
                4 * int.Parse(tfn.Substring(1, 1)) +
                3 * int.Parse(tfn.Substring(2, 1)) +
                7 * int.Parse(tfn.Substring(3, 1)) +
                5 * int.Parse(tfn.Substring(4, 1)) +
                8 * int.Parse(tfn.Substring(5, 1)) +
                6 * int.Parse(tfn.Substring(6, 1));

            decimal tfn8 = 9 * int.Parse(tfn.Substring(7, 1));

            if (tfn.Length == 8)
            {
                decimal tFNLg8WSum9 = 10 * int.Parse(tfn.Substring(7, 1));
                decimal tFNLg8WSum = tfn1To7Sum + tFNLg8WSum9;

                if ((tFNLg8WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else if (tfn.Length == 9)
            {
                decimal tfn9 = 10 * int.Parse(tfn.Substring(8, 1));
                decimal tFNLg9WSum = tfn1To7Sum + tfn8 + tfn9;

                if ((tFNLg9WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else
            {
                response = true;
            }

            return response;
        }


        private static decimal ConditionalValue(bool expression, decimal? trueVal, decimal? falseVal)
        {
             return expression ? trueVal.GetValueOrDefault() : falseVal.GetValueOrDefault();
        }

        private static decimal AsNumeric(string value)
        {
             decimal numberValue;
             decimal.TryParse(value, out numberValue);
             return numberValue;
        }

        private static bool RegexMatch(int? field, string expression, string flags="")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static bool RegexMatch(string field, string expression, string flags="")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static RegexOptions GetRegexOption(string flags)
        {
            RegexOptions options = RegexOptions.None;

            char[] characters = flags.ToCharArray();

            foreach (char character in characters)
            {
                switch (character)
                {
                    case 'i':                        
                        options = options | RegexOptions.IgnoreCase;                       
                        break;
                    case 'm':                        
                        options = options | RegexOptions.Multiline;                       
                        break;
                    case 's':                        
                            options = options | RegexOptions.Singleline;                       
                        break;
                    case 'n':                        
                            options = options | RegexOptions.ExplicitCapture;                       
                        break;
                    case 'x':                       
                            options = options | RegexOptions.IgnorePatternWhitespace;                       
                        break;
                }
            }

            return options;
        }

        private static string Substring(object field, int start, int length)
        {
            return field.ToString().Substring(start, length);
        }
        
        /// <summary>
        /// Returns an occurrence index as [occurrenceIndex] of occurrenceIndex > 0, otherwise the empty string
        /// </summary>
        /// <param name="occurrenceIndex">Index of the occurrence.</param>
        /// <returns>Occurrence in XPath [#] format</returns>
        public string OccurrenceIndex(int occurrenceIndex)
        {
            return occurrenceIndex > 0 ? "[" + occurrenceIndex + "]" : "";
        }

        #endregion // Functions

        public IITRPRFL2022 ConsumedReport { get {return report;} private set {report = value;}}

        private static Dictionary<string, ProcessMessageDocument> _processMessageDocuments = new Dictionary<string,ProcessMessageDocument>();				

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public List<ProcessMessageDocument> ValidateReport(IITRPRFL2022 reportIn)
        {
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            bool assertion;

            this.ConsumedReport = reportIn;

            // ------------------------------------------------------------------------------
            // Validations are now done in code.
            // This version supports full validation of a report - for those rules that have successfully parsed and could be generated.
            // Generated Validations include:
            //   - Validation logic in C#.
            //   - Production of errors in EBMS, SBR and SWS format.
            // 
            // The generation at this stage does not support the following - and has to be completed manually (later versions of the generation will do this for you)
            //   - Business Rules that did not parse.  The ESR team will help support you where this happens as an effort is being made to rectify these
            //   - You will get TODO tasks for those that the parser could not cope with
            // ------------------------------------------------------------------------------
                VRATOIITRPRFL000001();
                VRATOIITRPRFL000002();
                VRATOIITRPRFL000004();
                VRATOIITRPRFL000006();
                VRATOIITRPRFL000008();
                VRATOIITRPRFL000010();
                VRATOIITRPRFL000015();
                VRATOIITRPRFL000016();
                VRATOIITRPRFL000017();
                VRATOIITRPRFL000019();
                VRATOIITRPRFL000023();
                VRATOIITRPRFL000024();
                VRATOIITRPRFL000026();
                VRATOIITRPRFL000027();

            #region Repeating IITRPRFL4Collection  
            if (report.IITRPRFL4Collection != null)
            {
                int itemIndex2 = 0;
                foreach (string IITRPRFL4 in report.IITRPRFL4Collection)
                {
                    itemIndex2++;
                        VRATOIITRPRFL000003(IITRPRFL4, itemIndex2);
                        VRATOIITRPRFL000005(IITRPRFL4, itemIndex2);
                        VRATOIITRPRFL000007(IITRPRFL4, itemIndex2);
                        VRATOIITRPRFL000009(IITRPRFL4, itemIndex2);
                        VRATOIITRPRFL000018(IITRPRFL4, itemIndex2);
                        VRATOIITRPRFL000020(IITRPRFL4, itemIndex2);
                        VRATOIITRPRFL000025(IITRPRFL4, itemIndex2);
                    }
                }
        
                #endregion // Foreach loop

            #region Repeating report.Rp_Income_PayerCollection  
            if (report.Rp_Income_PayerCollection != null)
            {    
                int itemIndex3 = 0;
                foreach (IITRPRFL2022.Rp_Income_Payer payer in report.Rp_Income_PayerCollection)
                {
                    itemIndex3++;
                        VRATOIITRPRFL000011(payer, itemIndex3);
                        VRATOIITRPRFL000013(payer, itemIndex3);
                        VRATOIITRPRFL000014(payer, itemIndex3);
                    }
                }
        
                #endregion // Foreach loop

            #region Repeating report.Rp_Income_PayerCollection  
            if (report.Rp_Income_PayerCollection != null)
            {    
                int itemIndex3 = 0;
                foreach (IITRPRFL2022.Rp_Income_Payer payer in report.Rp_Income_PayerCollection)
                {
                    itemIndex3++;
        
                    #region Repeating IITRPRFL4Collection  
                    if (report.IITRPRFL4Collection != null)
                    {
                        int itemIndex2 = 0;
                        foreach (string IITRPRFL4 in report.IITRPRFL4Collection)
                        {
                            itemIndex2++;
                                VRATOIITRPRFL000012(payer, IITRPRFL4, itemIndex2);
                            }
                        }
                
                        #endregion // Foreach loop
                        }
                    }
            
                    #endregion // Foreach loop

            foreach (ProcessMessageDocument currentProcessMessage in response)
            {
                if (currentProcessMessage.Parameters != null)
                {
                    foreach (ProcessMessageParameter currentParameter in currentProcessMessage.Parameters)
                    {
                        if (string.IsNullOrEmpty(currentParameter.Name))
                        {
                            currentParameter.Name = _emptyParameterValue;
                        }

                        if (currentParameter.Name.Length > _maxParameterNameLength)
                        {
                            currentParameter.Name = currentParameter.Name.Substring(0, _maxParameterNameLength - 1);
                        }

                        if (string.IsNullOrEmpty(currentParameter.Value))
                        {
                            currentParameter.Value = _emptyParameterValue;
                        }

                        if (currentParameter.Value.Length > _maxParameterValueLength)
                        {
                            currentParameter.Value = currentParameter.Value.Substring(0, _maxParameterValueLength - 1);
                        }
                    }
                }
            }

            return response;
        }
				
			
    
      #region VR.ATO.IITRPRFL.000001
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000001()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.IITRPRFL.000001
            Tax Year must be current lodgment year
    
            Legacy Rule Format:
            (^IITRPRFL2 <> CurrentFinancialYear() - 1)

            Technical Business Rule Format:
            (^IITRPRFL2 <> CurrentFinancialYear() - 1)
    
            Data Elements:
    
            ^IITRPRFL2 = IITRPRFL:Rp:Report.TargetFinancial.Year
            */
            assertion = (report.IITRPRFL2.GetValueOrDefault() != CurrentFinancialYear() - 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITRPRFL.000001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Year of return is invalid",
                    LongDescription = @"The year of return must be the current lodgment year only. Substituted accounting periods (SAP) are not supported by this service.",
                    Location = "/tns:IITRPRFL/tns:Rp/tns:TargetFinancialY",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000001"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL2", Value = GetValueOrEmpty(report.IITRPRFL2) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.IITRPRFL.000002
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000002()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.IITRPRFL.000002
            Tax File Number must be a valid TFN
    
            Legacy Rule Format:
            (FailsTFNAlgorithm(^IITRPRFL3))

            Technical Business Rule Format:
            (FailsTFNAlgorithm(^IITRPRFL3))
    
            Data Elements:
    
            ^IITRPRFL3 = IITRPRFL:Rp:Identifiers.TaxFileNumber.Identifier
            */
            assertion = FailsTFNAlgorithm(report.IITRPRFL3);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.428016", Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax File Number has failed the algorithm check",
                    Location = "/tns:IITRPRFL/tns:Rp/tns:TaxFileNumberId",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000002"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL3", Value = report.IITRPRFL3 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.IITRPRFL.000003
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000003(string IITRPRFL4, int itemIndex2)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.IITRPRFL.000003
                    Where the Profile Compare Assessment Type requested is "Interest and dividends" then at least one Interest and Dividends amount must be provided
    
                    Legacy Rule Format:
                    (^IITRPRFL4 = "Interest and dividends" AND ^IITRPRFL15 = NULL AND ^IITRPRFL16 = NULL AND ^IITRPRFL17 = NULL AND ^IITRPRFL18 = NULL)
        
                    Technical Business Rule Format:
                    ^IITRPRFL4 = 'Interest and dividends' AND ^IITRPRFL15 = NULL AND ^IITRPRFL16 = NULL AND ^IITRPRFL17 = NULL AND ^IITRPRFL18 = NULL
            
                    Data Elements:
            
                    ^IITRPRFL4 = IITRPRFL:Rp:Report.ProfileCompareAssessmentType.Code
            
                    ^IITRPRFL15 = IITRPRFL:Rp:Income:Income.Interest.Gross.Amount
            
                    ^IITRPRFL16 = IITRPRFL:Rp:Income:Income.DividendsUnfranked.Amount
            
                    ^IITRPRFL17 = IITRPRFL:Rp:Income:Income.DividendsFranked.Amount
            
                    ^IITRPRFL18 = IITRPRFL:Rp:Income:IncomeTax.FrankingCredits.Amount
                    */
                    assertion = (IITRPRFL4 == @"Interest and dividends" && report.IITRPRFL15 == null && report.IITRPRFL16 == null && report.IITRPRFL17 == null && report.IITRPRFL18 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITRPRFL.000003", Severity = ProcessMessageSeverity.Error,
                            Description = @"Profile Compare Assessment Type details are incomplete",
                            LongDescription = @"Where the Profile Compare Assessment Type requested is ""Interest and dividends"" then at least one Interest and Dividends amount must be provided",
                            Location = "/tns:IITRPRFL/tns:Rp/tns:ProfileCompareAssessmentTypeCCollection/tns:ProfileCompareAssessmentTypeC[" + itemIndex2 + "]",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000003"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL4", Value = IITRPRFL4 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL15", Value = GetValueOrEmpty(report.IITRPRFL15) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL16", Value = GetValueOrEmpty(report.IITRPRFL16) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL17", Value = GetValueOrEmpty(report.IITRPRFL17) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL18", Value = GetValueOrEmpty(report.IITRPRFL18) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.IITRPRFL.000004
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000004()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.IITRPRFL.000004
            Where Interest and Dividend amounts are provided the Profile Compare Assessment Type must be set to "Interest and dividends"
    
            Legacy Rule Format:
            (CountOccurrence(^IITRPRFL4, ^IITRPRFL4  = "Interest and dividends") = 0 AND (^IITRPRFL15 > 0 OR ^IITRPRFL16 > 0 OR ^IITRPRFL17 > 0 OR ^IITRPRFL18 > 0))

            Technical Business Rule Format:
            (CountOccurrence(^IITRPRFL4, ^IITRPRFL4  = 'Interest and dividends') = 0 AND (^IITRPRFL15 > 0 OR ^IITRPRFL16 > 0 OR ^IITRPRFL17 > 0 OR ^IITRPRFL18 > 0))
    
            Data Elements:
    
            ^IITRPRFL4 = IITRPRFL:Rp:Report.ProfileCompareAssessmentType.Code
    
            ^IITRPRFL15 = IITRPRFL:Rp:Income:Income.Interest.Gross.Amount
    
            ^IITRPRFL16 = IITRPRFL:Rp:Income:Income.DividendsUnfranked.Amount
    
            ^IITRPRFL17 = IITRPRFL:Rp:Income:Income.DividendsFranked.Amount
    
            ^IITRPRFL18 = IITRPRFL:Rp:Income:IncomeTax.FrankingCredits.Amount
            */
            assertion = ((report.IITRPRFL4Collection == null ? 0 : report.IITRPRFL4Collection.Count(IITRPRFL4Repeat => IITRPRFL4Repeat == @"Interest and dividends")) == 0 && (report.IITRPRFL15.GetValueOrDefault() > 0 || report.IITRPRFL16.GetValueOrDefault() > 0 || report.IITRPRFL17.GetValueOrDefault() > 0 || report.IITRPRFL18.GetValueOrDefault() > 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITRPRFL.000004", Severity = ProcessMessageSeverity.Error,
                    Description = @"Profile Compare Assessment Type of ""Interest and dividends"" is missing",
                    LongDescription = @"Where Interest and Dividend amounts are provided, the Profile Compare Assessment Type must be set to ""Interest and dividends""",
                    Location = "/tns:IITRPRFL/tns:Rp/tns:ProfileCompareAssessmentTypeCCollection/tns:ProfileCompareAssessmentTypeC",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000004"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL15", Value = GetValueOrEmpty(report.IITRPRFL15) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL16", Value = GetValueOrEmpty(report.IITRPRFL16) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL17", Value = GetValueOrEmpty(report.IITRPRFL17) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL18", Value = GetValueOrEmpty(report.IITRPRFL18) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.IITRPRFL.000005
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000005(string IITRPRFL4, int itemIndex2)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.IITRPRFL.000005
                    Where the Profile Compare Assessment Type requested is "Work related expenses" then at least one Work Related Expense amount, and Salary and Wages Gross income must be provided
    
                    Legacy Rule Format:
                    (^IITRPRFL4 = "Work related expenses" AND ((^IITRPRFL20 = NULL AND ^IITRPRFL21 = NULL AND ^IITRPRFL22 = NULL AND ^IITRPRFL23 = NULL AND ^IITRPRFL24 = NULL) OR (Count(^IITRPRFL9) = 0)))
        
                    Technical Business Rule Format:
                    (^IITRPRFL4 = 'Work related expenses' AND ((^IITRPRFL20 = NULL AND ^IITRPRFL21 = NULL AND ^IITRPRFL22 = NULL AND ^IITRPRFL23 = NULL AND ^IITRPRFL24 = NULL) OR (Count(^IITRPRFL9) = 0)))
            
                    Data Elements:
            
                    ^IITRPRFL4 = IITRPRFL:Rp:Report.ProfileCompareAssessmentType.Code
            
                    ^IITRPRFL9 = IITRPRFL:Rp:Income:Payer
            
                    ^IITRPRFL20 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.Car.Amount
            
                    ^IITRPRFL21 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.Travel.Amount
            
                    ^IITRPRFL22 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.Clothing.Amount
            
                    ^IITRPRFL23 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.Education.Amount
            
                    ^IITRPRFL24 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.WorkRelatedOther.Amount
                    */
                    assertion = (IITRPRFL4 == @"Work related expenses" && (report.IITRPRFL20 == null && report.IITRPRFL21 == null && report.IITRPRFL22 == null && report.IITRPRFL23 == null && report.IITRPRFL24 == null || Count(report.Rp_Income_PayerCollectionCount) == 0));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITRPRFL.001005", Severity = ProcessMessageSeverity.Error,
                            Description = @"Profile Compare Assessment Type details are incomplete",
                            LongDescription = @"Where the Profile Compare Assessment Type requested is ""Work related expenses"" then at least one Work Related Expense amount, and Salary and Wages Gross income must be provided",
                            Location = "/tns:IITRPRFL/tns:Rp/tns:ProfileCompareAssessmentTypeCCollection/tns:ProfileCompareAssessmentTypeC[" + itemIndex2 + "]",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000005"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL4", Value = IITRPRFL4 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL20", Value = GetValueOrEmpty(report.IITRPRFL20) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL21", Value = GetValueOrEmpty(report.IITRPRFL21) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL22", Value = GetValueOrEmpty(report.IITRPRFL22) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL23", Value = GetValueOrEmpty(report.IITRPRFL23) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL24", Value = GetValueOrEmpty(report.IITRPRFL24) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.IITRPRFL.000006
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000006()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.IITRPRFL.000006
            Where Work related expense amounts are provided, the Profile Compare Assessment Type must be set to "Work related expenses"
    
            Legacy Rule Format:
            (CountOccurrence(^IITRPRFL4, ^IITRPRFL4 = "Work related expenses") = 0 AND (^IITRPRFL20 > 0 OR ^IITRPRFL21 > 0 OR ^IITRPRFL22 > 0 OR ^IITRPRFL23 > 0 OR ^IITRPRFL24 > 0))

            Technical Business Rule Format:
            (CountOccurrence(^IITRPRFL4, ^IITRPRFL4 = 'Work related expenses') = 0 AND (^IITRPRFL20 > 0 OR ^IITRPRFL21 > 0 OR ^IITRPRFL22 > 0 OR ^IITRPRFL23 > 0 OR ^IITRPRFL24 > 0))
    
            Data Elements:
    
            ^IITRPRFL4 = IITRPRFL:Rp:Report.ProfileCompareAssessmentType.Code
    
            ^IITRPRFL20 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.Car.Amount
    
            ^IITRPRFL21 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.Travel.Amount
    
            ^IITRPRFL22 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.Clothing.Amount
    
            ^IITRPRFL23 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.Education.Amount
    
            ^IITRPRFL24 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.WorkRelatedOther.Amount
            */
            assertion = ((report.IITRPRFL4Collection == null ? 0 : report.IITRPRFL4Collection.Count(IITRPRFL4Repeat => IITRPRFL4Repeat == @"Work related expenses")) == 0 && (report.IITRPRFL20.GetValueOrDefault() > 0 || report.IITRPRFL21.GetValueOrDefault() > 0 || report.IITRPRFL22.GetValueOrDefault() > 0 || report.IITRPRFL23.GetValueOrDefault() > 0 || report.IITRPRFL24.GetValueOrDefault() > 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITRPRFL.000006", Severity = ProcessMessageSeverity.Error,
                    Description = @"Profile Compare Assessment Type of ""Work related expenses"" is missing",
                    LongDescription = @"Where Work related expense amounts are provided, the Profile Compare Assessment Type must be set to ""Work related expenses""",
                    Location = "/tns:IITRPRFL/tns:Rp/tns:ProfileCompareAssessmentTypeCCollection/tns:ProfileCompareAssessmentTypeC",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000006"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL20", Value = GetValueOrEmpty(report.IITRPRFL20) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL21", Value = GetValueOrEmpty(report.IITRPRFL21) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL22", Value = GetValueOrEmpty(report.IITRPRFL22) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL23", Value = GetValueOrEmpty(report.IITRPRFL23) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL24", Value = GetValueOrEmpty(report.IITRPRFL24) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.IITRPRFL.000007
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000007(string IITRPRFL4, int itemIndex2)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.IITRPRFL.000007
                    Where the Profile Compare Assessment Type requested is "Other deductions" then Total Other deductions amount, and Salary and Wages Gross income fields must be provided
    
                    Legacy Rule Format:
                    (^IITRPRFL4 = "Other deductions" AND (^IITRPRFL25 = NULL OR (Count(^IITRPRFL9) = 0)))
        
                    Technical Business Rule Format:
                    (^IITRPRFL4 = 'Other deductions' AND (^IITRPRFL25 = NULL OR (Count(^IITRPRFL9) = 0)))
            
                    Data Elements:
            
                    ^IITRPRFL4 = IITRPRFL:Rp:Report.ProfileCompareAssessmentType.Code
            
                    ^IITRPRFL9 = IITRPRFL:Rp:Income:Payer
            
                    ^IITRPRFL25 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.UncategorisedOther.Amount
                    */
                    assertion = (IITRPRFL4 == @"Other deductions" && (report.IITRPRFL25 == null || Count(report.Rp_Income_PayerCollectionCount) == 0));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITRPRFL.001007", Severity = ProcessMessageSeverity.Error,
                            Description = @"Profile Compare Assessment Type details are incomplete",
                            LongDescription = @"Where the Profile Compare Assessment Type requested is ""Other deductions"" then Total Other deductions amount, and Salary and Wages Gross income fields must be provided",
                            Location = "/tns:IITRPRFL/tns:Rp/tns:ProfileCompareAssessmentTypeCCollection/tns:ProfileCompareAssessmentTypeC[" + itemIndex2 + "]",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000007"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL4", Value = IITRPRFL4 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL25", Value = GetValueOrEmpty(report.IITRPRFL25) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.IITRPRFL.000008
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000008()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.IITRPRFL.000008
            Where Total Other deductions amount is provided the Profile Compare Assessment Type must be set to "Other deductions"
    
            Legacy Rule Format:
            (CountOccurrence(^IITRPRFL4, ^IITRPRFL4 = "Other deductions") = 0 AND ^IITRPRFL25 > 0)

            Technical Business Rule Format:
            (CountOccurrence(^IITRPRFL4, ^IITRPRFL4 = 'Other deductions') = 0 AND ^IITRPRFL25 > 0)
    
            Data Elements:
    
            ^IITRPRFL4 = IITRPRFL:Rp:Report.ProfileCompareAssessmentType.Code
    
            ^IITRPRFL25 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.UncategorisedOther.Amount
            */
            assertion = ((report.IITRPRFL4Collection == null ? 0 : report.IITRPRFL4Collection.Count(IITRPRFL4Repeat => IITRPRFL4Repeat == @"Other deductions")) == 0 && report.IITRPRFL25.GetValueOrDefault() > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITRPRFL.000008", Severity = ProcessMessageSeverity.Error,
                    Description = @"Profile Compare Assessment Type of ""Other deductions"" is missing",
                    LongDescription = @"Where Total Other deductions amount is provided, the Profile Compare Assessment Type must be set to ""Other deductions""",
                    Location = "/tns:IITRPRFL/tns:Rp/tns:ProfileCompareAssessmentTypeCCollection/tns:ProfileCompareAssessmentTypeC",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000008"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL25", Value = GetValueOrEmpty(report.IITRPRFL25) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.IITRPRFL.000009
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000009(string IITRPRFL4, int itemIndex2)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.IITRPRFL.000009
                    Where the Profile Compare Assessment Type requested is "Cost of managing tax affairs" then Cost of managing tax affairs - Other expense amount must be provided
    
                    Legacy Rule Format:
                    (^IITRPRFL4 = "Cost of managing tax affairs" AND ^IITRPRFL26 = NULL)
        
                    Technical Business Rule Format:
                    ^IITRPRFL4 = 'Cost of managing tax affairs' AND ^IITRPRFL26 = NULL
            
                    Data Elements:
            
                    ^IITRPRFL4 = IITRPRFL:Rp:Report.ProfileCompareAssessmentType.Code
            
                    ^IITRPRFL26 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.TaxAffairManagementOther.Amount
                    */
                    assertion = (IITRPRFL4 == @"Cost of managing tax affairs" && report.IITRPRFL26 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITRPRFL.000009", Severity = ProcessMessageSeverity.Error,
                            Description = @"Profile Compare Assessment Type details are incomplete",
                            LongDescription = @"Where the Profile Compare Assessment Type requested is ""Cost of managing tax affairs"" then Cost of managing tax affairs - Other expenses amount must be provided",
                            Location = "/tns:IITRPRFL/tns:Rp/tns:ProfileCompareAssessmentTypeCCollection/tns:ProfileCompareAssessmentTypeC[" + itemIndex2 + "]",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000009"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL4", Value = IITRPRFL4 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL26", Value = GetValueOrEmpty(report.IITRPRFL26) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.IITRPRFL.000010
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000010()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.IITRPRFL.000010
            Where Cost of managing tax affairs - Other amount is provided, Profile Compare Assessment Type must be set to "Cost of managing tax affairs"
    
            Legacy Rule Format:
            (CountOccurrence(^IITRPRFL4, ^IITRPRFL4 = "Cost of managing tax affairs") = 0 AND ^IITRPRFL26 > 0)

            Technical Business Rule Format:
            (CountOccurrence(^IITRPRFL4, ^IITRPRFL4 = 'Cost of managing tax affairs') = 0 AND ^IITRPRFL26 > 0)
    
            Data Elements:
    
            ^IITRPRFL4 = IITRPRFL:Rp:Report.ProfileCompareAssessmentType.Code
    
            ^IITRPRFL26 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.TaxAffairManagementOther.Amount
            */
            assertion = ((report.IITRPRFL4Collection == null ? 0 : report.IITRPRFL4Collection.Count(IITRPRFL4Repeat => IITRPRFL4Repeat == @"Cost of managing tax affairs")) == 0 && report.IITRPRFL26.GetValueOrDefault() > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITRPRFL.000010", Severity = ProcessMessageSeverity.Error,
                    Description = @"Profile Compare Assessment Type of ""Cost of managing tax affairs"" is missing",
                    LongDescription = @"Where Cost of managing tax affairs - Other amount is provided, Profile Compare Assessment Type must be set to ""Cost of managing tax affairs""",
                    Location = "/tns:IITRPRFL/tns:Rp/tns:ProfileCompareAssessmentTypeCCollection/tns:ProfileCompareAssessmentTypeC",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000010"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL26", Value = GetValueOrEmpty(report.IITRPRFL26) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.IITRPRFL.000011
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000011(IITRPRFL2022.Rp_Income_Payer payer, int itemIndex3)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.IITRPRFL.000011
                    Payer's Australian Business Number must be a valid ABN
    
                    Legacy Rule Format:
                    (FailsABNAlgorithm(^IITRPRFL10))
        
                    Technical Business Rule Format:
                    (FailsABNAlgorithm(^IITRPRFL10))
            
                    Data Elements:
            
                    ^IITRPRFL10 = IITRPRFL:Rp:Income:Payer:Identifiers.AustralianBusinessNumber.Identifier
                    */
                    assertion = FailsABNAlgorithm(payer.IITRPRFL10);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITRPRFL.000011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Payer's Australian Business Number (ABN) has failed the ABN algorithm check",
                            Location = "/tns:IITRPRFL/tns:Rp/tns:Income/tns:PayerCollection/tns:Payer" + OccurrenceIndex(payer.OccurrenceIndex) + "/tns:AustralianBusinessNumberId",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000011"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL10", Value = GetValueOrEmpty(payer.IITRPRFL10) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.IITRPRFL.000012
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000012(IITRPRFL2022.Rp_Income_Payer payer, string IITRPRFL4, int itemIndex2)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
                    
                            /*  VR.ATO.IITRPRFL.000012
                            When a positive Salary and Wages Gross income amount has been provided, and the Profile Compare Assessment Type requested is "Work Related Expenses" either the Payer's Australian Business Number or Payer's Withholder Payer Number must be provided
            
                            Legacy Rule Format:
                            (^IITRPRFL12 > 0 AND ^IITRPRFL10 = NULL AND ^IITRPRFL11 = NULL AND ^IITRPRFL4 = "Work related expenses")
                
                            Technical Business Rule Format:
                            ^IITRPRFL12 > 0 AND ^IITRPRFL10 = NULL AND ^IITRPRFL11 = NULL AND ^IITRPRFL4 = 'Work related expenses'
                    
                            Data Elements:
                    
                            ^IITRPRFL10 = IITRPRFL:Rp:Income:Payer:Identifiers.AustralianBusinessNumber.Identifier
                    
                            ^IITRPRFL4 = IITRPRFL:Rp:Report.ProfileCompareAssessmentType.Code
                    
                            ^IITRPRFL11 = IITRPRFL:Rp:Income:Payer:Identifiers.WithholdingPayerNumber.Identifier
                    
                            ^IITRPRFL12 = IITRPRFL:Rp:Income:Payer:Income.SalaryOrWages.Amount
                            */
                            assertion = (payer.IITRPRFL12.GetValueOrDefault() > 0 && payer.IITRPRFL10 == null && payer.IITRPRFL11 == null && IITRPRFL4 == @"Work related expenses");
                            if (assertion)
                            {
                                processMessage = new ProcessMessageDocument()
                                {
                                    Code = "CMN.ATO.IITRPRFL.001012", Severity = ProcessMessageSeverity.Error,
                                    Description = @"Either ABN or WPN must be present",
                                    LongDescription = @"When a positive Salary and Wages Gross income amount has been provided, and the Profile Compare Assessment Type requested is ""Work Related Expenses"" either the Payer's Australian Business Number or Payer's Withholder Payer Number must be provided",
                                    Location = "/tns:IITRPRFL/tns:Rp/tns:Income/tns:PayerCollection/tns:Payer" + OccurrenceIndex(payer.OccurrenceIndex) + "/tns:AustralianBusinessNumberId",
                                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000012"} },
                                };
                                processMessage.Parameters.Add(new ProcessMessageParameter
                                    { Name = "IITRPRFL12", Value = GetValueOrEmpty(payer.IITRPRFL12) });
                    
                                processMessage.Parameters.Add(new ProcessMessageParameter
                                    { Name = "IITRPRFL10", Value = GetValueOrEmpty(payer.IITRPRFL10) });
                    
                                processMessage.Parameters.Add(new ProcessMessageParameter
                                    { Name = "IITRPRFL11", Value = GetValueOrEmpty(payer.IITRPRFL11) });
                    
                                processMessage.Parameters.Add(new ProcessMessageParameter
                                    { Name = "IITRPRFL4", Value = IITRPRFL4 });
                    
                                response.Add(processMessage);
                            }
                      }
                      #endregion 
    
      #region VR.ATO.IITRPRFL.000013
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000013(IITRPRFL2022.Rp_Income_Payer payer, int itemIndex3)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.IITRPRFL.000013
                    Either Payer's Australian Business Number (ABN) or Payer's Withholder Payer Number (WPN) can be populated, but not both
    
                    Legacy Rule Format:
                    (^IITRPRFL10 <> NULL AND ^IITRPRFL11 <> NULL)
        
                    Technical Business Rule Format:
                    ^IITRPRFL10 <> NULL AND ^IITRPRFL11 <> NULL
            
                    Data Elements:
            
                    ^IITRPRFL10 = IITRPRFL:Rp:Income:Payer:Identifiers.AustralianBusinessNumber.Identifier
            
                    ^IITRPRFL11 = IITRPRFL:Rp:Income:Payer:Identifiers.WithholdingPayerNumber.Identifier
                    */
                    assertion = (payer.IITRPRFL10 != null && payer.IITRPRFL11 != null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITRPRFL.000013", Severity = ProcessMessageSeverity.Error,
                            Description = @"ABN and WPN must not both be provided",
                            LongDescription = @"Either Payer's Australian Business Number (ABN) or Payer's Withholder Payer Number (WPN) can be populated, but not both",
                            Location = "/tns:IITRPRFL/tns:Rp/tns:Income/tns:PayerCollection/tns:Payer" + OccurrenceIndex(payer.OccurrenceIndex) + "/tns:AustralianBusinessNumberId",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000013"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL10", Value = GetValueOrEmpty(payer.IITRPRFL10) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL11", Value = GetValueOrEmpty(payer.IITRPRFL11) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.IITRPRFL.000014
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000014(IITRPRFL2022.Rp_Income_Payer payer, int itemIndex3)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.IITRPRFL.000014
                    Payer's Withholder Payer Number must be a valid WPN
    
                    Legacy Rule Format:
                    (FailsWPNAlgorithm(^IITRPRFL11))
        
                    Technical Business Rule Format:
                    (FailsWPNAlgorithm(^IITRPRFL11))
            
                    Data Elements:
            
                    ^IITRPRFL11 = IITRPRFL:Rp:Income:Payer:Identifiers.WithholdingPayerNumber.Identifier
                    */
                    assertion = FailsTFNAlgorithm(payer.IITRPRFL11);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.200012", Severity = ProcessMessageSeverity.Error,
                            Description = @"Invalid WPN",
                            Location = "/tns:IITRPRFL/tns:Rp/tns:Income/tns:PayerCollection/tns:Payer" + OccurrenceIndex(payer.OccurrenceIndex) + "/tns:WithholdingPayerNumberId",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000014"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL11", Value = GetValueOrEmpty(payer.IITRPRFL11) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.IITRPRFL.000015
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000015()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.IITRPRFL.000015
            Tax Agent's Australian Business Number must be a valid ABN.
    
            Legacy Rule Format:
            (FailsABNAlgorithm(^IITRPRFL28))

            Technical Business Rule Format:
            (FailsABNAlgorithm(^IITRPRFL28))
    
            Data Elements:
    
            ^IITRPRFL28 = IITRPRFL:Int:Identifiers.AustralianBusinessNumber.Identifier
            */
            assertion = FailsABNAlgorithm(report.IITRPRFL28);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITRPRFL.000015", Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax Agent's Australian Business Number (ABN) has failed the ABN algorithm check",
                    Location = "/tns:IITRPRFL/tns:Int/tns:AustralianBusinessNumberId",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000015"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL28", Value = report.IITRPRFL28 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.IITRPRFL.000016
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000016()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.IITRPRFL.000016
            Tax Agent's Number must be a valid TAN
    
            Legacy Rule Format:
            (FailsTANAlgorithm(^IITRPRFL29))

            Technical Business Rule Format:
            (FailsTANAlgorithm(^IITRPRFL29))
    
            Data Elements:
    
            ^IITRPRFL29 = IITRPRFL:Int:Identifiers.TaxAgentNumber.Identifier
            */
            assertion = FailsTANAlgorithm(report.IITRPRFL29);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410009", Severity = ProcessMessageSeverity.Error,
                    Description = @"The Tax Agent number has failed the Tax Agent number algorithm check",
                    Location = "/tns:IITRPRFL/tns:Int/tns:TaxAgentNumberId",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000016"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL29", Value = report.IITRPRFL29 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.IITRPRFL.000017
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000017()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.IITRPRFL.000017
            Duplicate values cannot be provided for Profile Compare Assessment Type
    
            Legacy Rule Format:
            (HasDuplicateValues(^IITRPRFL4))

            Technical Business Rule Format:
            (HasDuplicateValues(^IITRPRFL4))
    
            Data Elements:
    
            ^IITRPRFL4 = IITRPRFL:Rp:Report.ProfileCompareAssessmentType.Code
            */
            assertion = HasDuplicateValues(report.IITRPRFL4Collection);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITRPRFL.000017", Severity = ProcessMessageSeverity.Error,
                    Description = @"Profile Compare Assessment Type code must be unique",
                    Location = "/tns:IITRPRFL/tns:Rp/tns:ProfileCompareAssessmentTypeCCollection/tns:ProfileCompareAssessmentTypeC" + OccurrenceIndex(DuplicateValueIndex(report.IITRPRFL4Collection) + 1) + "",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000017"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.IITRPRFL.000018
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000018(string IITRPRFL4, int itemIndex2)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.IITRPRFL.000018
                    Where the Profile Compare Assessment Type requested is "Rental" then Rent - Number of properties, and at least one Rental amount must be provided (Rent - Gross rent; Rent - Interest deductions; Rent - Capital works deductions; Rent - Other rental deductions)
    
                    Legacy Rule Format:
                    (^IITRPRFL4 = "Rental" AND (^IITRPRFL35 = NULL OR (^IITRPRFL31 = NULL AND ^IITRPRFL32 = NULL AND ^IITRPRFL33 = NULL AND ^IITRPRFL34 = NULL)))
        
                    Technical Business Rule Format:
                    (^IITRPRFL4 = 'Rental' AND (^IITRPRFL35 = NULL OR (^IITRPRFL31 = NULL AND ^IITRPRFL32 = NULL AND ^IITRPRFL33 = NULL AND ^IITRPRFL34 = NULL)))
            
                    Data Elements:
            
                    ^IITRPRFL4 = IITRPRFL:Rp:Report.ProfileCompareAssessmentType.Code
            
                    ^IITRPRFL31 = IITRPRFL:Rp:Income:IncomeTaxDeduction:Income.RealEstateProperty.Rental.Amount
            
                    ^IITRPRFL32 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.RentalIncomeInterestDeductions.Amount
            
                    ^IITRPRFL33 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.CapitalWorksDeductions.Amount
            
                    ^IITRPRFL34 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            
                    ^IITRPRFL35 = IITRPRFL:Rp:Income:IncomeTaxDeduction:RealEstateProperty.Properties.Count
                    */
                    assertion = (IITRPRFL4 == @"Rental" && (report.IITRPRFL35 == null || report.IITRPRFL31 == null && report.IITRPRFL32 == null && report.IITRPRFL33 == null && report.IITRPRFL34 == null));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITRPRFL.000018", Severity = ProcessMessageSeverity.Error,
                            Description = @"Profile Compare Assessment Type details are incomplete",
                            LongDescription = @"Where the Profile Compare Assessment Type requested is ""Rental"" then Rent - Number of properties, and at least one Rental amount must be provided (Rent - Gross rent; Rent - Interest deductions; Rent - Capital works deductions; Rent - Other rental deductions)",
                            Location = "/tns:IITRPRFL/tns:Rp/tns:ProfileCompareAssessmentTypeCCollection/tns:ProfileCompareAssessmentTypeC[" + itemIndex2 + "]",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000018"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL4", Value = IITRPRFL4 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL35", Value = report.IITRPRFL35.GetValueOrDefault().ToString() });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL31", Value = GetValueOrEmpty(report.IITRPRFL31) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL32", Value = GetValueOrEmpty(report.IITRPRFL32) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL33", Value = GetValueOrEmpty(report.IITRPRFL33) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL34", Value = GetValueOrEmpty(report.IITRPRFL34) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.IITRPRFL.000019
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000019()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.IITRPRFL.000019
            Where Rent - Number of properties, or any Rental amounts are provided, the Profile Compare Assessment Type must be set to "Rental"
    
            Legacy Rule Format:
            (CountOccurrence(^IITRPRFL4, ^IITRPRFL4 = "Rental") = 0 AND (^IITRPRFL35 > 0 OR ^IITRPRFL31 > 0 OR ^IITRPRFL32 > 0 OR ^IITRPRFL33 > 0 OR ^IITRPRFL34 > 0))

            Technical Business Rule Format:
            (CountOccurrence(^IITRPRFL4, ^IITRPRFL4 = 'Rental') = 0 AND (^IITRPRFL35 > 0 OR ^IITRPRFL31 > 0 OR ^IITRPRFL32 > 0 OR ^IITRPRFL33 > 0 OR ^IITRPRFL34 > 0))
    
            Data Elements:
    
            ^IITRPRFL4 = IITRPRFL:Rp:Report.ProfileCompareAssessmentType.Code
    
            ^IITRPRFL31 = IITRPRFL:Rp:Income:IncomeTaxDeduction:Income.RealEstateProperty.Rental.Amount
    
            ^IITRPRFL32 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.RentalIncomeInterestDeductions.Amount
    
            ^IITRPRFL33 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.CapitalWorksDeductions.Amount
    
            ^IITRPRFL34 = IITRPRFL:Rp:Income:IncomeTaxDeduction:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
    
            ^IITRPRFL35 = IITRPRFL:Rp:Income:IncomeTaxDeduction:RealEstateProperty.Properties.Count
            */
            assertion = ((report.IITRPRFL4Collection == null ? 0 : report.IITRPRFL4Collection.Count(IITRPRFL4Repeat => IITRPRFL4Repeat == @"Rental")) == 0 && (report.IITRPRFL35.GetValueOrDefault() > 0 || report.IITRPRFL31.GetValueOrDefault() > 0 || report.IITRPRFL32.GetValueOrDefault() > 0 || report.IITRPRFL33.GetValueOrDefault() > 0 || report.IITRPRFL34.GetValueOrDefault() > 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITRPRFL.000019", Severity = ProcessMessageSeverity.Error,
                    Description = @"Profile Compare Assessment Type of ""Rental"" is missing",
                    LongDescription = @"Where Rent - Number of properties, or any Rental amounts are provided, the Profile Compare Assessment Type must be set to ""Rental""",
                    Location = "/tns:IITRPRFL/tns:Rp/tns:ProfileCompareAssessmentTypeCCollection/tns:ProfileCompareAssessmentTypeC",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000019"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL35", Value = report.IITRPRFL35.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL31", Value = GetValueOrEmpty(report.IITRPRFL31) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL32", Value = GetValueOrEmpty(report.IITRPRFL32) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL33", Value = GetValueOrEmpty(report.IITRPRFL33) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL34", Value = GetValueOrEmpty(report.IITRPRFL34) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.IITRPRFL.000020
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000020(string IITRPRFL4, int itemIndex2)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.IITRPRFL.000020
                    Where the Profile Compare Assessment Type requested is "Work related expenses" then Occupation Details must be provided (Main salary and wage occupation description, and Occupation code)
    
                    Legacy Rule Format:
                    (^IITRPRFL4 = "Work related expenses" AND ^IITRPRFL6 = NULLORBLANK AND ^IITRPRFL7 = NULL)
        
                    Technical Business Rule Format:
                    ^IITRPRFL4 = 'Work related expenses' AND ^IITRPRFL6 = BLANK AND ^IITRPRFL7 = NULL
            
                    Data Elements:
            
                    ^IITRPRFL4 = IITRPRFL:Rp:Report.ProfileCompareAssessmentType.Code
            
                    ^IITRPRFL6 = IITRPRFL:Rp:PersonDemographicDetailsOccupation:PersonDemographicDetails.Occupation.Description
            
                    ^IITRPRFL7 = IITRPRFL:Rp:PersonDemographicDetailsOccupation:PersonDemographicDetails.Occupation.Code
                    */
                    assertion = (IITRPRFL4 == @"Work related expenses" && string.IsNullOrWhiteSpace(report.IITRPRFL6) == true && report.IITRPRFL7 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITRPRFL.000020", Severity = ProcessMessageSeverity.Error,
                            Description = @"Profile Compare Assessment Type details are incomplete",
                            LongDescription = @"Where the Profile Compare Assessment Type requested is ""Work related expenses"" then Occupation Details must be provided (Main salary and wage occupation description, and Occupation code)",
                            Location = "/tns:IITRPRFL/tns:Rp/tns:ProfileCompareAssessmentTypeCCollection/tns:ProfileCompareAssessmentTypeC[" + itemIndex2 + "]",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000020"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL4", Value = IITRPRFL4 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL6", Value = report.IITRPRFL6 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL7", Value = report.IITRPRFL7 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.IITRPRFL.000023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000023()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.IITRPRFL.000023
            Business name cannot contain 'space hyphen space'
    
            Legacy Rule Format:
            Contains(^IITRPRFL39, ' - ')

            Technical Business Rule Format:
            Contains(^IITRPRFL39, ' - ')
    
            Data Elements:
    
            ^IITRPRFL39 = IITRPRFL:Rp:Income:SoleTrader:OrganisationNameDetails.OrganisationalName.Text
            */
            assertion = (report.IITRPRFL39 == null ? false : report.IITRPRFL39.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITRPRFL.000023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Business name of main business must not have space hyphen space combinations",
                    Location = "/tns:IITRPRFL/tns:Rp/tns:Income/tns:SoleTrader/tns:OrganisationNameDetailsOrganisationalNameT",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000023"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL39", Value = report.IITRPRFL39 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.IITRPRFL.000024
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000024()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.IITRPRFL.000024
            Sole trader's Australian Business Number must be a valid ABN.
    
            Legacy Rule Format:
            (FailsABNAlgorithm(^IITRPRFL40))

            Technical Business Rule Format:
            (FailsABNAlgorithm(^IITRPRFL40))
    
            Data Elements:
    
            ^IITRPRFL40 = IITRPRFL:Rp:Income:SoleTrader:Identifiers.AustralianBusinessNumber.Identifier
            */
            assertion = FailsABNAlgorithm(report.IITRPRFL40);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITRPRFL.000024", Severity = ProcessMessageSeverity.Error,
                    Description = @"Sole trader's Australian Business Number (ABN) has failed the ABN algorithm check.",
                    Location = "/tns:IITRPRFL/tns:Rp/tns:Income/tns:SoleTrader/tns:AustralianBusinessNumberId",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000024"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL40", Value = GetValueOrEmpty(report.IITRPRFL40) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.IITRPRFL.000025
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000025(string IITRPRFL4, int itemIndex2)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.IITRPRFL.000025
                    Where the Profile Compare Assessment Type requested is "Sole trader", then Sole trader details must be provided (Description of main business or professional activity, Industry code of main business, Postcode of main business address, Business name of main business, Total business income, Total business expenses, Total expense reconciliation adjustments and Payments to associated persons)
    
                    Legacy Rule Format:
                    ^IITRPRFL4 = "Sole trader" AND ^IITRPRFL1000 = NULL
        
                    Technical Business Rule Format:
                    ^IITRPRFL4 = 'Sole trader' AND ^IITRPRFL1000 = NULL
            
                    Data Elements:
            
                    ^IITRPRFL4 = IITRPRFL:Rp:Report.ProfileCompareAssessmentType.Code
            
                    ^IITRPRFL1000 = IITRPRFL:Rp:Income:SoleTrader
                    */
                    assertion = (IITRPRFL4 == @"Sole trader" && report.Rp_Income_SoleTraderCollectionExists == false);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITRPRFL.000025", Severity = ProcessMessageSeverity.Error,
                            Description = @"Profile Compare Assessment Type details are incomplete.",
                            LongDescription = @"Where the Profile Compare Assessment Type requested is ""Sole trader"", then Sole trader details must be provided (Description of main business or professional activity, Industry code of main business, Postcode of main business address, Business name of main business, Total business income, Total business expenses, Total expense reconciliation adjustments and Payments to associated persons)",
                            Location = "/tns:IITRPRFL/tns:Rp/tns:ProfileCompareAssessmentTypeCCollection/tns:ProfileCompareAssessmentTypeC[" + itemIndex2 + "]",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000025"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITRPRFL4", Value = IITRPRFL4 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.IITRPRFL.000026
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000026()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.IITRPRFL.000026
            Where Sole trader assessment details are provided, the Profile Compare Assessment Type must be set to "Sole trader".
    
            Legacy Rule Format:
            (CountOccurrence(^IITRPRFL4, ^IITRPRFL4 = "Sole trader") = 0 AND ^IITRPRFL1000 <> NULL)

            Technical Business Rule Format:
            (CountOccurrence(^IITRPRFL4, ^IITRPRFL4 = 'Sole trader') = 0 AND ^IITRPRFL1000 <> NULL)
    
            Data Elements:
    
            ^IITRPRFL4 = IITRPRFL:Rp:Report.ProfileCompareAssessmentType.Code
    
            ^IITRPRFL1000 = IITRPRFL:Rp:Income:SoleTrader
            */
            assertion = ((report.IITRPRFL4Collection == null ? 0 : report.IITRPRFL4Collection.Count(IITRPRFL4Repeat => IITRPRFL4Repeat == @"Sole trader")) == 0 && report.Rp_Income_SoleTraderCollectionExists != false);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITRPRFL.000026", Severity = ProcessMessageSeverity.Error,
                    Description = @"Profile Compare Assessment Type of ""Sole trader"" is missing.",
                    LongDescription = @"Where Sole trader assessment details are provided, the Profile Compare Assessment Type must be set to ""Sole trader"".",
                    Location = "/tns:IITRPRFL/tns:Rp/tns:ProfileCompareAssessmentTypeCCollection/tns:ProfileCompareAssessmentTypeC",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000026"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.IITRPRFL.000027
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOIITRPRFL000027()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.IITRPRFL.000027
            Postcode of main business address of the sole trader should be within the valid range of 0200 to 9999.
    
            Legacy Rule Format:
            (^IITRPRFL44 <> NULL) AND (^IITRPRFL44 < 0200 OR ^IITRPRFL44 > 9999)

            Technical Business Rule Format:
            (^IITRPRFL44 <> NULL) AND (^IITRPRFL44 < 0200 OR ^IITRPRFL44 > 9999)
    
            Data Elements:
    
            ^IITRPRFL44 = IITRPRFL:Rp:Income:SoleTrader:AddressDetails.Postcode.Text
            */
            assertion = (report.IITRPRFL44 != null && (AsNumeric(report.IITRPRFL44) < 200 || AsNumeric(report.IITRPRFL44) > 9999));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.428041", Severity = ProcessMessageSeverity.Error,
                    Description = @"For an Australian address, postcode is invalid. Valid range is 0200 - 9999",
                    Location = "/tns:IITRPRFL/tns:Rp/tns:Income/tns:SoleTrader/tns:AddressDetailsPostcodeT",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.IITRPRFL.000027"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITRPRFL44", Value = report.IITRPRFL44 });
    
                response.Add(processMessage);
            }
      }
      #endregion 

    }
} 
